## Polymer 2 Decorators <a name="decorators"></a>
Decorators can be used to describe element properties, observers, etc.

### @customElement(tagName) <a name="decorators-customElement"></a>

Sets the tag name of the custom element. The decorator is applied to the `class`. Tag names must include a `-` as per WebComponents specs.

Example of a `<my-element>`:

```Typescript
@customElement('my-element')
class MyElement extends Polymer.Element {

}
```

### @property(propertyConfig) <a name="decorators-property"></a>

Creates a Polymer property. It can be applied to a **static** class properties only.

The parameter `propertyConfig` is a map object that sets the options for the property:

```Typescript
{
    type?: enum;                     // Boolean, Date, Number, String, Array or Object
    reflectToAttribute?: boolean;    // should the property reflect to attribute
    readonly?: boolean;              // marks property as read-only
    notify?: boolean;                // if set to true, will trigger "propname-changed"
    computed?: string;               // computed function name (as string)
}
```

Examples:
```Typescript
@property
static foo;
```

```TypeScript
// type: is specified for property deserialization.
// Used when it is passed to another elements in html attribute
@property({type: Number})
static initialValue = 42;
```

```TypeScript
@property({readonly: true, computed: '_computeValue(foo, initialValue)'})
static computedReadOnlyValue = 0;

_computeValue(foo, initialValue) {
  return foo.toString() + initialValue;
}
```

### @computed(propList, computedFn) <a name="decorators-computed"></a>

Makes a property computed. It can be applied to a **static** class properties only.

```TypeScript
@property
static one = 'one';

@property
static two = 'two';

@property
@computed('one, two', (one, two) => one + two)
static three;
```

### @observe(propList) <a name="decorators-observe"></a>

Sets an observer function for a single property or a list of properties.

If observing a single property, the function should be of the type `function(newValue, oldValue)`.

If observing multiple properties (comma separated), the function receives only the new values,
in the same order of the list.

```TypeScript
// single property observer
@observe('name')
nameChanged(newName, oldName) {
   // ...
}
```

```TypeScript
// multiple property observer
@observe('firstname, lastname')
fullnameChanged(newFirstName, newLastName) {
   // ...
}
```

### @observeOnValue(prop) <a name="decorators-observe-on-value"></a>

Same as @observe decorator, but the function is called when the newValue is not null or undefined.

Only single property can be observed.

### @observeSinceValue(prop) <a name="decorators-observe-since-value"></a>

Same as @observe decorator, but the function is called on the next change, since property was initialized with any value (oldValue !== undefined).

Only single property can be observed.

### @eventListener(eventType, options) <a name="decorators-event-listener"></a>

Sets an event listener for the element and binds it to a function.
Options &mdash;
[addEventListener](https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/addEventListener)
options.

```TypeScript
@eventListener('click')
onClick(e) {
   e.stopPropagation();
}
```

### @statePath(statePath) <a name="decorators-state-path"></a>

Use with [polymer-redux](https://github.com/tur-nr/polymer-redux/tree/polymer-2)

Assigns a Polymer property to reflect the Redux app state.

```Typescript
  @property
  @statePath('objects.total')
  static totalItems;
```

### @routerStateParameter(stateParameter, options) <a name="decorators-router-state-parameter"></a>

Use with [polymer-2-ui-router](https://github.com/mkorenko/polymer-2-ui-router)

Assigns a Polymer property to reflect current ui-router state parameter.

The parameter `options` is a map object that sets the options for the property:

```Typescript
{
    reflectToState?: boolean;    // should the property be reflected back to state on change
}
```

```Typescript
  @property
  @routerStateParameter('page')
  static currentPage;
```